import 'dart:convert';
import 'package:flutter/material.dart';
import 'package:http/http.dart' as http;
import 'package:walk_2_school/main.dart';
import 'package:walk_2_school/standardized_widgets.dart';
import 'package:walk_2_school/misc.dart';
import 'package:google_fonts/google_fonts.dart';

class CreateAccountPage extends StatefulWidget {
  const CreateAccountPage({super.key});

  @override
  _CreateAccountPageState createState() => _CreateAccountPageState();
}

class _CreateAccountPageState extends State<CreateAccountPage> {
  final _formKey = GlobalKey<FormState>();
  String _username = '';
  String _password = '';
  bool _isLoading = false;

  Future<void> _createAccount(String username, String password) async {
    final response = await http.post(
      Uri.parse('https://walk-2-school-backend.vercel.app/create-account'),
      headers: {'Content-Type': 'application/json'},
      body: json.encode({
        'username': username,
        'password': hashPassword(password),
      }),
    );

    if (response.body.contains("Account created")) {
      ScaffoldMessenger.of(navigatorKey.currentContext!).showSnackBar(
        const SnackBar(content: Text('Account created.')),
      );
      Navigator.pop(navigatorKey.currentContext!);
    } else {
      if (response.body.contains("Username already exists")) {
        ScaffoldMessenger.of(navigatorKey.currentContext!).showSnackBar(
          const SnackBar(content: Text('Username already exists.')),
        );
      } else if (response.body.contains("Invalid Username Format")) {
        ScaffoldMessenger.of(navigatorKey.currentContext!).showSnackBar(
          const SnackBar(content: Text('Invalid Username Format.')),
        );
      } else {
        ScaffoldMessenger.of(navigatorKey.currentContext!).showSnackBar(
          const SnackBar(content: Text('Account creation failed.')),
        );
      }
    }
  }

  void back() {
    Navigator.pop(context);
  }

  @override
  Widget build(BuildContext context) {
    double width = MediaQuery.of(context).size.width;
    double height = MediaQuery.of(context).size.height;
    return Scaffold(
      backgroundColor: StandardizedWidgets.background,
      bottomNavigationBar: Container(
          color: StandardizedWidgets.green,
          child: SizedBox(
              height: height / 13,
              width: MediaQuery.of(context).size.width,
              child: Column(children: [
                StandardizedWidgets().spacer(height: height / 150),
                SizedBox(
                  height: height / 24,
                  child: IconButton(
                    color: Colors.white,
                    icon: const Icon(Icons.arrow_back_rounded),
                    onPressed: () {
                      Navigator.pop(context);
                    },
                  ),
                ),
                Text(
                  'Back to Login Page',
                  style: GoogleFonts.montserrat(
                    fontSize: 12,
                    color: Colors.white,
                    fontWeight: FontWeight.w600,
                  ),
                )
              ]))),
      body: SingleChildScrollView(
        child: Padding(
          padding: const EdgeInsets.all(16.0),
          child: Column(
            mainAxisAlignment: MainAxisAlignment.start,
            children: [
              StandardizedWidgets().spacer(height: height / 30),
              SizedBox(
                height: height / 5,
                width: width / 1.2,
                child: FittedBox(
                  fit: BoxFit.fill,
                  child: StandardizedWidgets().largeLogo(),
                ),
              ),
              StandardizedWidgets().spacer(height: height / 30),
              StandardizedWidgets()
                  .titleText('Create your Walk2School account here!'),
              StandardizedWidgets().spacer(height: height / 30),
              Form(
                key: _formKey,
                child: Column(
                  mainAxisAlignment: MainAxisAlignment.center,
                  children: [
                    //refactor all text form fields with reg text fields
                    TextFormField(
                      style: StandardizedWidgets().typingFont,
                      decoration: const InputDecoration(
                          hintText: 'Email',
                          filled: true,
                          fillColor: Colors.white,
                          enabledBorder: OutlineInputBorder(
                            borderSide: BorderSide(color: Colors.white),
                            borderRadius: BorderRadius.all(Radius.circular(20)),
                          ),
                          focusedBorder: OutlineInputBorder(
                              borderSide: BorderSide(color: Colors.white),
                              borderRadius:
                                  BorderRadius.all(Radius.circular(20)))),
                      validator: (value) {
                        if (value == null || value.isEmpty) {
                          return 'Please enter a username.';
                        }
                        return null;
                      },
                      onSaved: (value) {
                        _username = value ?? '';
                      },
                    ),

                    StandardizedWidgets().spacer(height: height / 80),
                    TextFormField(
                      style: StandardizedWidgets().typingFont,
                      decoration: const InputDecoration(
                          hintText: 'Password',
                          filled: true,
                          fillColor: Colors.white,
                          enabledBorder: OutlineInputBorder(
                            borderSide: BorderSide(color: Colors.white),
                            borderRadius: BorderRadius.all(Radius.circular(20)),
                          ),
                          focusedBorder: OutlineInputBorder(
                              borderSide: BorderSide(color: Colors.white),
                              borderRadius:
                                  BorderRadius.all(Radius.circular(20)))),
                      validator: (value) {
                        if (value == null || value.isEmpty) {
                          return 'Please enter a password.';
                        }
                        return null;
                      },
                      onSaved: (value) {
                        _password = value ?? '';
                      },
                    ),

                    StandardizedWidgets().spacer(height: height / 35),
                    _isLoading
                        ? const CircularProgressIndicator()
                        : ElevatedButton(
                            style: ButtonStyle(
                                textStyle: MaterialStateProperty.all<TextStyle>(
                                    StandardizedWidgets().loginFont),
                                backgroundColor:
                                    MaterialStateProperty.all<Color>(
                                        StandardizedWidgets.green),
                                fixedSize: MaterialStateProperty.all<Size>(
                                    Size(width / 1.3, height / 20)),
                                shape: MaterialStateProperty.all<
                                    RoundedRectangleBorder>(
                                  const RoundedRectangleBorder(
                                      borderRadius: BorderRadius.all(
                                          Radius.circular(20))),
                                )),
                            onPressed: () {
                              if (_formKey.currentState!.validate()) {
                                _formKey.currentState!.save();
                                setState(() {
                                  _isLoading = true;
                                });
                                _createAccount(_username, _password).then(
                                    (_) => setState(() => _isLoading = false));
                              }
                            },
                            child: const Text('Create Account'),
                          ),
                  ],
                ),
              ),
            ],
          ),
        ),
      ),
    );
  }
}
