import 'dart:convert';
import 'package:flutter/material.dart';
import 'package:google_fonts/google_fonts.dart';
import 'package:http/http.dart' as http;
import 'package:walk_2_school/standardized_widgets.dart';
import 'package:walk_2_school/main.dart';

class ForgotPasswordPage extends StatefulWidget {
  const ForgotPasswordPage({super.key});

  @override
  _ForgotPasswordPageState createState() => _ForgotPasswordPageState();
}

class _ForgotPasswordPageState extends State<ForgotPasswordPage> {
  final _formKey = GlobalKey<FormState>();
  String _username = '';
  bool _isLoading = false;

  Future<void> _submitForgotPassword(String username) async {
    final response = await http.post(
      Uri.parse('https://walk-2-school-backend.vercel.app/forgot-password'),
      headers: {'Content-Type': 'application/json'},
      body: json.encode({
        'username': username,
      }),
    );

    if (response.body.contains("Email sent")) {
      ScaffoldMessenger.of(navigatorKey.currentContext!).showSnackBar(
        const SnackBar(
            content: Text(
                'An email containing a link to reset your password has been sent.')),
      );
      Navigator.pop(navigatorKey.currentContext!);
    } else {
      if (response.body.contains("Username does not exist")) {
        ScaffoldMessenger.of(navigatorKey.currentContext!).showSnackBar(
          const SnackBar(
              content: Text('No registered account with that email.')),
        );
      } else if (response.body.contains("Error sending email")) {
        ScaffoldMessenger.of(navigatorKey.currentContext!).showSnackBar(
          const SnackBar(content: Text('Error sending email.')),
        );
      } else if (response.body.contains("Error modifying user")) {
        ScaffoldMessenger.of(navigatorKey.currentContext!).showSnackBar(
          const SnackBar(content: Text('Error adding token.')),
        );
      } else {
        ScaffoldMessenger.of(navigatorKey.currentContext!).showSnackBar(
          const SnackBar(content: Text('Password reset failed.')),
        );
      }
    }
  }

  @override
  Widget build(BuildContext context) {
    double width = MediaQuery.of(context).size.width;
    double height = MediaQuery.of(context).size.height;
    return Scaffold(
      backgroundColor: StandardizedWidgets.background,
      bottomNavigationBar: Container(
          color: StandardizedWidgets.green,
          child: SizedBox(
              height: height / 13,
              width: MediaQuery.of(context).size.width,
              child: Column(children: [
                StandardizedWidgets().spacer(height: height / 150),
                SizedBox(
                  height: height / 24,
                  child: IconButton(
                    color: Colors.white,
                    icon: const Icon(Icons.arrow_back_rounded),
                    onPressed: () {
                      Navigator.pop(context);
                    },
                  ),
                ),
                Text(
                  'Back to Login Page',
                  style: GoogleFonts.montserrat(
                    fontSize: 12,
                    color: Colors.white,
                    fontWeight: FontWeight.w600,
                  ),
                )
              ]))),

      //appBar: AppBar(title: const Text('Forgot Password')),
      body: SingleChildScrollView(
        child: Padding(
          padding: const EdgeInsets.all(16.0),
          child: Column(
            mainAxisAlignment: MainAxisAlignment.start,
            children: [
              StandardizedWidgets().spacer(height: height / 30),
              SizedBox(
                height: height / 5,
                width: width / 1.2,
                child: FittedBox(
                  fit: BoxFit.fill,
                  child: StandardizedWidgets().largeLogo(),
                ),
              ),
              StandardizedWidgets().spacer(height: 30),
              Row(
                mainAxisAlignment: MainAxisAlignment.start,
                children: [
                  Expanded(
                    child: Column(
                      crossAxisAlignment: CrossAxisAlignment.start,
                      children: [
                        StandardizedWidgets().titleText('Forgot\nPassword?'),
                        StandardizedWidgets().spacer(height: height / 150),
                        StandardizedWidgets().subscriptText(
                            'Enter the email associated with your account and we\'ll send you a link to reset your password.',
                            color: StandardizedWidgets.yellow),
                      ],
                    ),
                  ),
                ],
              ),
              StandardizedWidgets().spacer(height: height / 30),
              Form(
                key: _formKey,
                child: Column(
                  children: [
                    //refactor all text form fields with reg text fields

                    TextFormField(
                      style: StandardizedWidgets().typingFont,
                      decoration: const InputDecoration(
                          hintText: 'Email',
                          filled: true,
                          fillColor: Colors.white,
                          enabledBorder: OutlineInputBorder(
                            borderSide: BorderSide(color: Colors.white),
                            borderRadius: BorderRadius.all(Radius.circular(20)),
                          ),
                          focusedBorder: OutlineInputBorder(
                              borderSide: BorderSide(color: Colors.white),
                              borderRadius:
                                  BorderRadius.all(Radius.circular(20)))),
                      validator: (value) {
                        if (value == null || value.isEmpty) {
                          return 'Please enter the email associated with your account';
                        }
                        return null;
                      },
                      onSaved: (value) {
                        _username = value ?? '';
                      },
                    ),
                    StandardizedWidgets().spacer(height: height / 60),
                    _isLoading
                        ? const CircularProgressIndicator()
                        : ElevatedButton(
                            style: ButtonStyle(
                                textStyle: MaterialStateProperty.all<TextStyle>(
                                    StandardizedWidgets().loginFont),
                                backgroundColor:
                                    MaterialStateProperty.all<Color>(
                                        StandardizedWidgets.green),
                                fixedSize: MaterialStateProperty.all<Size>(
                                    Size(width / 1.3, height / 20)),
                                shape: MaterialStateProperty.all<
                                    RoundedRectangleBorder>(
                                  const RoundedRectangleBorder(
                                      borderRadius: BorderRadius.all(
                                          Radius.circular(20))),
                                )),
                            onPressed: () {
                              if (_formKey.currentState!.validate()) {
                                _formKey.currentState!.save();
                                setState(() {
                                  _isLoading = true;
                                });
                                _submitForgotPassword(_username).then(
                                    (_) => setState(() => _isLoading = false));
                              }
                            },
                            child: const Text('Submit'),
                          ),
                  ],
                ),
              ),
            ],
          ),
        ),
      ),
    );
  }
}
