import 'dart:async';

import 'dart:math';

import 'package:flutter/material.dart';
import 'package:geolocator/geolocator.dart';
import 'package:intl/intl.dart';
import 'package:percent_indicator/percent_indicator.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:walk_2_school/pages/whos_walking_page.dart';
import 'package:walk_2_school/standardized_widgets.dart';
import 'package:walk_2_school/misc.dart';
import 'package:walk_2_school/user_data.dart';
import 'package:walk_2_school/pages/leaderboard_page.dart';
import 'package:walk_2_school/bottom_nav_bar.dart';
import 'package:walk_2_school/stat_card.dart';

class HomePage extends StatefulWidget {
  static final GlobalKey<NavigatorState> navigatorKey =
      GlobalKey<NavigatorState>();

  const HomePage({super.key});
  @override
  _HomePageState createState() => _HomePageState();
}

// bool isVisible = true;
// List<FlSpot> meters = [];
// double xmax = 1;
// double ymax = 1;

class _HomePageState extends State<HomePage> {
  final bool testMode = false;
  late List<String> timeStamps;
  late List<String> points;
  late List<String> distances;
  late List<String> streaks;
  int _startingPoints = 0;
  Position? _startPosition;
  Position? _currentPosition;
  double _startDistance = 0;
  double _remainingDistance = 0;
  double _progress = 0;
  bool _isWalking = false;
  bool _isInitializing = false;
  bool _speedExceeded = false;
  UserData _currUserData = UserData.empty();
  double _sessionDistance = 0;
  Stopwatch _stopwatch = Stopwatch();
  StreamSubscription<Position>? _positionStream;
  static const double schoolLatitude = 32.911447564043634;
  static const double schoolLongitude = 35.281764785755406;
  DateTime _lastHeartbeat = DateTime.fromMillisecondsSinceEpoch(0);

  bool _confirmEndWalk = false;
  @override
  void initState() {
    super.initState();
    _loadUserData();
  }

  void _loadUserData() async {
    _currUserData = await UserData.retrieveLocally();
    _currUserData.checkStreakReset();
    // store the user data locally
    await _currUserData.storeLocally();
    setState(() {
      _currUserData = _currUserData;
    });
  }

  storeStatistics() async {
    int endingPoints = _currUserData.currPoints;

    int diffPoints = max(endingPoints - _startingPoints, 0);
    SharedPreferences prefs = await SharedPreferences.getInstance();
    String timeStamp = DateTime.now().toIso8601String();
    timeStamps = prefs.getStringList('timeStampList') ?? [];
    timeStamps.add(timeStamp);
    prefs.setStringList('timeStampList', timeStamps);

    distances = prefs.getStringList('distanceList') ?? [];
    distances.add(_progress.toString());
    prefs.setStringList('distanceList', distances);

    points = prefs.getStringList('pointsList') ?? [];
    points.add(diffPoints.toString());
    prefs.setStringList('pointsList', points);

    streaks = prefs.getStringList('streakList') ?? [];
    streaks.add(_currUserData.currStreak.toString());
    prefs.setStringList('streakList', streaks);

    // Update _currUserData.currPoints and corresponding SharedPreferences value
    _currUserData.currPoints += diffPoints;
    prefs.setInt('currPoints', _currUserData.currPoints);
  }

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
  }

  @override
  Widget build(BuildContext context) {
    // final progress = Provider.of<ProgressProvider>(context, listen: false);
    // progress.setProgress(_sessionDistance);
    // final progressProvider =
    //     Provider.of<ProgressProvider>(context, listen: false);
    // progressProvider.setProgress(_sessionDistance);
    final formatter = NumberFormat.compact();
    formatter.maximumFractionDigits = 1;
    return Scaffold(
      appBar: AppBar(
        backgroundColor: const Color(0xFF687A3E),
        centerTitle: true,
        title: Text(
          getHelloMessage(),
          style: const TextStyle(
            fontFamily: 'Montserrat',
            fontWeight: FontWeight.bold,
          ),
        ),
        actions: [
          IconButton(
            color: Colors.white,
            icon: const Icon(Icons.supervised_user_circle),
            onPressed: () {
              //navigate to statistics

              Navigator.push(
                context,
                MaterialPageRoute(
                    builder: (context) => const WhosWalkingPage()),
                // (Route<dynamic> route) => false,
              );
            },
          ),
          IconButton(
            icon: const Icon(
              Icons.leaderboard,
              color: Colors.white,
            ),
            onPressed: () {
              // swap to leaderboard page
              Navigator.push(
                context,
                MaterialPageRoute(
                  builder: (context) => const LeaderboardPage(),
                ),
              );
            },
          ),
        ],
      ),
      body: Center(
        heightFactor: 1.1,
        child: SingleChildScrollView(
          child: Column(
            mainAxisAlignment: MainAxisAlignment.center,
            children: <Widget>[
              //StandardizedWidgets().lineChart(),
              CircularPercentIndicator(
                radius: 150.0,
                lineWidth: 30.0,
                percent: _progress.abs(),
                center: Column(
                  mainAxisAlignment: MainAxisAlignment.center,
                  children: [
                    Text(
                      '${(_progress.abs() * 100).toStringAsFixed(0)}%',
                      style: const TextStyle(
                          fontSize: 50, fontFamily: 'Montserrat'),
                    ),
                    Text(
                      '${(_remainingDistance / 10).floor() * 10} m',
                      style: const TextStyle(
                          fontSize: 24, fontFamily: 'Montserrat'),
                    ),
                    const Text(
                      'Remaining',
                      style: TextStyle(
                          fontSize: 16,
                          color: Colors.grey,
                          fontFamily: 'Montserrat'),
                    )
                  ],
                ),
                progressColor: Colors.green,
                backgroundColor: Colors.grey,
                circularStrokeCap: CircularStrokeCap.round,
              ),
              const SizedBox(height: 20),
              SizedBox(
                width: 350.0,
                height: 50.0,
                child: ElevatedButton(
                  style: ElevatedButton.styleFrom(
                    backgroundColor: _isWalking || _isInitializing
                        ? StandardizedWidgets.yellow
                        : StandardizedWidgets.green,
                    shape: const RoundedRectangleBorder(
                      borderRadius: BorderRadius.all(Radius.circular(10)),
                    ),
                  ),
                  onPressed: () {
                    _startWalkPressed();
                  },
                  child: _isInitializing
                      ? const Text('Initializing...',
                          style: TextStyle(
                            fontFamily: 'Montserrat',
                            fontWeight: FontWeight.bold,
                            fontSize: 20,
                          ))
                      : _isWalking
                          ? const Text('Stop Walk',
                              style: TextStyle(
                                fontFamily: 'Montserrat',
                                fontWeight: FontWeight.bold,
                                fontSize: 20,
                              ))
                          : const Text('Start Walk',
                              style: TextStyle(
                                fontFamily: 'Montserrat',
                                fontWeight: FontWeight.bold,
                                fontSize: 20,
                              )),
                ),
              ),
              const SizedBox(height: 20),
              Padding(
                padding: const EdgeInsets.only(left: 16.0, right: 16.0),
                child: Container(
                  //constraints: const BoxConstraints(maxWidth: 400),
                  child: Row(
                    mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                    children: [
                      StatCard(
                          icon: Icons.directions_walk,
                          label: 'Distance',
                          value: '${_sessionDistance.toStringAsFixed(0)} m',
                          description: "Current Walk Distance (meters)",
                          iconColor: StandardizedWidgets.green),
                      StatCard(
                          icon: Icons.explore,
                          label: 'All Time',
                          value:
                              '${formatter.format(_currUserData.allTimeDistance)}m',
                          description: 'All Time Distance (meters)',
                          iconColor: Colors.blue),
                    ],
                  ),
                ),
              ),
              Padding(
                padding: const EdgeInsets.only(left: 16.0, right: 16.0),
                child: Row(
                  mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                  children: [
                    // AnimatedSwitcher(duration: Duration(seconds: 2),child: ),
                    GestureDetector(
                      child: StatCard(
                        icon: Icons.whatshot_outlined,
                        label: 'Streak',
                        value: '${_currUserData.currStreak}',
                        description:
                            'Consecutive days walked. Acts as a multiplier for points!',
                        iconColor: StandardizedWidgets.yellow,
                      ),
                      onTap: () {
                        // setState(() {
                        //   isVisible = !isVisible;
                        // });
                      },
                    ),
                    StatCard(
                      icon: Icons.grade,
                      label: 'Points',
                      value: formatter.format(_currUserData.currPoints),
                      description: 'Points that can be spent',
                      iconColor: StandardizedWidgets.purple,
                    ),
                  ],
                ),
              ),
            ],
          ),
        ),
      ),
      bottomNavigationBar: bottomNavBar(context, 'home', _isWalking),
    );
  }

  resetState() {
    _confirmEndWalk = false;
    setState(() {
      _isWalking = false;
      _startPosition = null;
      _currentPosition = null;
      _startDistance = 0;
      _remainingDistance = 0;
      _sessionDistance = 0;
      _progress = 0;
      _isInitializing = false;

      _currUserData = _currUserData;
      _stopwatch.stop();
      _stopwatch.reset();
    });

    _positionStream!.cancel();
    _positionStream = null;
  }

  showSpeedExceededDialog() {
    return StandardizedWidgets().basicAppDialog(
        context,
        'Speed Exceeded',
        'We have detected that you may have used a vehicle during your commute. Any data collected will be discarded.',
        'OK');
  }

  showWalkTooShortDialog() {
    return StandardizedWidgets().basicAppDialog(context, 'Walk Too Short',
        'Sorry, your walk was too short to be counted.', 'OK');
  }

  showNotCloseEnoughDialog() {
    return StandardizedWidgets().basicAppDialog(
        context,
        'Not Close Enough',
        'Sorry, you were not close enough to the destination for your walk to be counted (100m).',
        'OK');
  }

  endWalk() async {
    if (_speedExceeded) {
      showSpeedExceededDialog();
      // restore userdata from sharedprefs
      _currUserData = await UserData.retrieveLocally();
      _currUserData.speedExceededCount += 1;
      await _currUserData.storeLocally();
      await UserData.sendUserDataToServer();
      print("Sent user data to server");
      resetState();
      return;
    }

    // check if walk was long enough
    if (_sessionDistance < 50 && !testMode) {
      showWalkTooShortDialog();
      // restore userdata from sharedprefs
      _currUserData = await UserData.retrieveLocally();
      await _currUserData.storeLocally();
      await UserData.sendUserDataToServer();
      print("Sent user data to server");
      resetState();
      return;
    }

    // check if end of walk is within 100m of end
    if (_remainingDistance > 100 && !testMode) {
      showNotCloseEnoughDialog();
      // restore userdata from sharedprefs
      _currUserData = await UserData.retrieveLocally();
      await _currUserData.storeLocally();
      await UserData.sendUserDataToServer();
      print("Sent user data to server");
      resetState();
      return;
    }

    // check if the time is after noon
    if (DateTime.now().hour >= 12 && !testMode) {
      StandardizedWidgets().basicAppDialog(context, "Too Late",
          "You can't walk after noon. Come back tomorrow to walk again!", "OK");
      // restore userdata from sharedprefs
      _currUserData = await UserData.retrieveLocally();
      await _currUserData.storeLocally();
      await UserData.sendUserDataToServer();
      print("Sent user data to server");
      resetState();
      return;
    }

    _currUserData.walkDurations.add(_stopwatch.elapsed.inSeconds);
    _currUserData.walkDistances.add(_sessionDistance.toInt());
    _currUserData.dailyAvgSpeeds
        .add((_sessionDistance / _stopwatch.elapsed.inSeconds));

    //here the walk meets the requirements

    // check if should add to streak

    _currUserData.updateStreak();

    // push userdata to sharedprefs
    await storeStatistics();
    await _currUserData.storeLocally();
    await UserData.sendUserDataToServer();
    print("Sent user data to server");

    resetState();

    return;
  }

  _startWalkPressed() async {
    if (_isInitializing) {
      return;
    }

    if (_isWalking) {
      // prompt the user to reconsider if they don't meet the requirements for a valid walk
      if (_remainingDistance > 100 && !_confirmEndWalk) {
        StandardizedWidgets().confirmDialog(
          context,
          "Confirm End Walk",
          "You are not close enough to the destination (100m) to have your walk counted. Are you sure you want to end your walk?",
          "Yes",
          "No",
          () {
            print("Action confirmed");
            _confirmEndWalk = true;
            _startWalkPressed();
          },
        );
        return;
      }
    }

    // check if they have already walked today
    if (_currUserData.lastWalkUpdate.day == DateTime.now().day && !testMode) {
      StandardizedWidgets().basicAppDialog(
          context,
          "Already Walked Today",
          "You have already walked today. Come back tomorrow to walk again!",
          "OK");
      return;
    }

    // check if today is a weekend, israel uses friday and sat as weekend
    if (DateTime.now().weekday == 5 || DateTime.now().weekday == 6) {
      // 5 is friday, 6 is saturday
      StandardizedWidgets().basicAppDialog(
          context,
          "Weekend",
          "You can't walk on the weekend. Come back on a weekday to walk again!",
          "OK");
      return;
    }

    // make sure the local time is before noon to start a walk
    if (DateTime.now().hour >= 12 && !testMode) {
      StandardizedWidgets().basicAppDialog(context, "Too Late",
          "You can't walk after noon. Come back tomorrow to walk again!", "OK");
      return;
    }

    setState(() {
      _isWalking = !_isWalking;
    });

    if (!_isWalking) {
      await endWalk();
      return;
    }

    setState(() {
      _isInitializing = true;
    });

    // start stopwatch
    _stopwatch.start();

    _startingPoints = _currUserData.currPoints;
    // add a baseline of 100 points for the walk
    _currUserData.currPoints += 100;
    _startPosition = await Geolocator.getCurrentPosition(
        desiredAccuracy: LocationAccuracy.high);
    print("Got start position");
    _currentPosition = _startPosition;
    _startDistance = Geolocator.distanceBetween(_startPosition!.latitude,
        _startPosition!.longitude, schoolLatitude, schoolLongitude);
    _remainingDistance = Geolocator.distanceBetween(_currentPosition!.latitude,
        _currentPosition!.longitude, schoolLatitude, schoolLongitude);

    _positionStream = Geolocator.getPositionStream(
        locationSettings: const LocationSettings(
      accuracy: LocationAccuracy.high,
      distanceFilter: 10,
    )).listen((Position position) {
      setState(() {
        double currDistanceFromSchool = Geolocator.distanceBetween(
            position.latitude,
            position.longitude,
            schoolLatitude,
            schoolLongitude);
        if (currDistanceFromSchool > _startDistance) {
          _startDistance = currDistanceFromSchool;
        }

        double distFromLastPos = Geolocator.distanceBetween(
            _currentPosition!.latitude,
            _currentPosition!.longitude,
            position.latitude,
            position.longitude);
        _sessionDistance += distFromLastPos.toInt();
        _currUserData.allTimeDistance += distFromLastPos.toInt();
        _currUserData.addPoints(
            (distFromLastPos * (_currUserData.currStreak + 1)).toInt());

        _currentPosition = position;
        _remainingDistance = Geolocator.distanceBetween(
            _currentPosition!.latitude,
            _currentPosition!.longitude,
            schoolLatitude,
            schoolLongitude);
        _progress = (_startDistance - _remainingDistance) / _startDistance;
        // progressProvider
        double currSpeed = position.speed;
        if (currSpeed > 11.176) {
          // 25 mph invalidate the walk
          _speedExceeded = true;
        }
      });
      // check if need to send heartbeat
      if (DateTime.now().difference(_lastHeartbeat).inSeconds >= 60) {
        _lastHeartbeat = DateTime.now();
        UserData.sendHeartbeat(position.latitude, position.longitude);
      }
    });
    print('Walk started');
    setState(() {
      _isInitializing = false;
    });
  }

  String _getSuffix(int value) {
    List<String> suffixes = ['', 'k', 'm', 'b', 't'];

    int index = 0;
    while (value >= 1000 && index < suffixes.length - 1) {
      value ~/= 1000;
      index++;
    }

    return suffixes[index];
  }
}
